<?php

class bbPress_Content_Filters extends WECR_Content_Filters{

	function __construct(){
		add_filter( 'bbp_current_user_can_access_create_topic_form', array( $this, 'topic_form_access' ), 10, 1 );
		add_filter( 'bbp_after_has_topics_parse_args', array( $this, 'topic_access' ), 10 );
		add_filter( 'bbp_get_reply_content', array( $this, 'hide_reply_content' ), 10, 2 );
		add_action( 'template_redirect', array( $this, 'apply_feedback_messages' ) );
	}

	/**
	 * Hides the topic form if you don't have access
	 *
	 * @since 0.0.1
	 * @author SFNdesign, Curtis McHale
	 * @access public
	 *
	 * @param bool  $value      Base value of bbp_current_user_can_access_create_topic_form
	 *
	 * @return bool             Our modified conditional value
	 *
	 * @uses $this->forum_is_restricted()           Returns true if the parent forum is restricted
	 * @uses $this->user_has_admin_access()         Returns true if the current user has the read cap assigned
	 * @uses $this->user_purchased_product()        Returns true if the user has purchased the product that we're restricted by
	 */
	public function topic_form_access( $value ){

		if ( ! $this->forum_is_restricted( get_the_ID() ) ) return true;
		if ( $this->user_has_admin_access() ) return true;
		if ( $this->user_purchased_product( get_the_ID()  ) ) return true;

		return false;

	} // topic_access

	/**
	 * Returns true if the parent forum of the topic is restricted.
	 *
	 * @param int   $post_id    required    post_id of the topic we're checking
	 *
	 * @return bool                         True if the forum is restricted
	 *
	 * @global $post                            Big WP post global ya'll
	 * @uses get_post_type()                    Returns post_type given id or object
	 * @uses get_post_meta()                    returns post meta given post_id and key
	 * @uses $this->check_ancestors()           Loops through to check any ancestors of the post_object for restriction
	 */
	private function forum_is_restricted( $post_id ){

		global $post;

		$ancestors = $post->ancestors;
		$restricted = false;

		if ( get_post_type( $post_id ) === 'forum' ){
			$forum_id = absint( $post_id );
		} else {
			$forum_id = get_post_meta( $post_id, '_bbp_forum_id', true );
		}

		if ( $ancestors ){
			$restricted = $this->check_ancestors( $ancestors );
		} else {

			$is_restricted = get_post_meta( absint( $forum_id ), '_wecr_easy_restrict_content', true );

			if ( isset( $is_restricted ) && ! empty( $is_restricted ) ){
				$restricted = true;
			}

		}

		return (bool) $restricted;

	} // forum_is_restricted

	/**
	 * Hides the reply content from the forums
	 *
	 * @since 1.2.2
	 * @author SFNdesign, Curtis McHale
	 * @access public
	 *
	 * @param string        $content        optional        Reply content
	 * @param int           $reply_id       optional        ID of the reply
	 * @global $post                                        Big WP post global
	 * @uses $this->user_has_admin_access()                 Returns true if the user has admin access
	 * @uses $this->user_purchased_product()                Returns true if the user has purchased the product
	 * @uses $this->get_restricted_message()                Returns the restricted message if none given
	 * @return string
	 */
	public function hide_reply_content( $content, $reply_id ){

		$ancestors = get_post_ancestors( $reply_id );
		$access = false;

		if ( $this->user_has_admin_access() ) return $content;

		if ( is_array( $ancestors ) && ! empty( $ancestors ) ){
			foreach( $ancestors as $a ){
				if ( $this->user_purchased_product( absint( $a ) ) ){
					$access = true;
					break;
				}
			}
		}

		if ( true === $access ){
			return $content;
		} else {
			$content = apply_filters( 'wecr_bbp_hide_reply_content_global', $this->get_restricted_message( "This content is restricted to purchasers" ) );
			return wp_kses_post( $content );
		}

	} // hide_reply_content

	/**
	 * Restricts the topics in a forum to users that have purchased a product
	 *
	 * @since 0.0.1
	 * @author SFNdesign, Curtis McHale
	 * @access public
	 *
	 * @param $query
	 * @return array
	 *
	 * @uses $this->user_has_admin_access()         Returns true if the current user can read posts
	 * @uses bbp_is_single_forum()                  Returns true if this is a single forum
	 * @uses $this->user_purchased_product()        Returns true if the user has purchased this product
	 * @uses bbp_get_forum_id()                     Returns the id of the forum
	 */
	public function topic_access( $query ){

		$is_restricted = true;

		if ( $this->user_has_admin_access() || ! bbp_is_single_forum() ) return $query;

		if ( $this->user_purchased_product( bbp_get_forum_id() ) ){
			$is_restricted = false;
		}

		if( $is_restricted ){
			// @todo this gives us some debug output that I need to find and kill
			return array(); // empty query
		} else {
			return $query;
		}

	} // topic_access

	/**
	 * Apply the filtered feedback messages to the forums
	 *
	 * @since 0.0.1
	 * @author SFNdesign, Curtis McHale
	 * @access public
	 * @hattip Pippins Plugins
	 *
	 * @global  $user_ID
	 * @return  void
	 */
	public function apply_feedback_messages() {

		global $user_ID;

		if ( bbp_is_single_topic() ) {
			add_filter( 'gettext', array( $this, 'topic_feedback_messages' ), 20, 3 );
		} else if ( bbp_is_single_forum() && $this->forum_is_restricted( bbp_get_forum_id() ) ) {
			add_filter( 'gettext', array( $this, 'forum_feedback_messages' ), 20, 3 );
		}

	} // apply_feedback_messages

	/**
	 * Override topic feedback message
	 *
	 * @since 0.0.1
	 * @author SFNdesign, Curtis McHale
	 * @access public
	 * @hattip Pippins Plugins
	 *
	 * @param string        $translated_text    the original feedback text
	 * @param string        $text
	 * @param string        $domain             the textdomain to use
	 *
	 * @return string       $translated_text    the filtered feedback text
	 */
	public function topic_feedback_messages( $translated_text, $text, $domain ) {

		switch ( $translated_text ) {
			case 'You cannot reply to this topic.':
				$translated_text_out = apply_filters( 'wecr_restricted_content_message_cant_reply', $this->get_restricted_message() );
				break;
			default:
				$translated_text_out = $translated_text;
				break;
		}

		$text_out = apply_filters( 'wecr_bbp_topic_message_global', $translated_text_out, $translated_text );

		return wp_kses_post( $text_out );

	} // topic_feedback_messages

	/**
	 * Override forum feedback message
	 *
	 * @since 0.0.1
	 * @author SFNdesign, Curtis McHale
	 * @access public
	 * @hattip Pippins Plugins
	 *
	 * @param string    $translated_text        the original feedback text
	 * @param string    $text
	 * @param string    $domain                 the textdomain to use
	 *
	 * @return string   $translated_text        the filtered feedback text
	 */
	public function forum_feedback_messages( $translated_text, $text, $domain ) {

		switch ( $translated_text ) {
			case 'Oh bother! No topics were found here!':
				$translated_text_out = apply_filters( 'wecr_restricted_content_message_no_topics_found', $this->get_restricted_message() );
				break;
			case 'You cannot create new topics at this time.':
				$translated_text_out = apply_filters( 'wecr_restricted_content_message_cant_create_topics', $this->get_restricted_message() );
				break;
			case 'You must be logged in to create new topics.':
				$message = 'Topic creation is restricted to customers with a current purchase.';
				$translated_text_out = apply_filters( 'wecr_restricted_content_message_logged_in_create_topics', $this->get_restricted_message( $message ) );
				break;
			default:
				$translated_text_out = $translated_text;
				break;
		}

		$text_out = apply_filters( 'wecr_bbp_forum_message_global', $translated_text_out, $translated_text );

		return wp_kses_post( $text_out );

	} // forum_feedback_messages

}
$GLOBALS['bbpress_content_filters'] = new bbPress_Content_Filters();
